/* gcalc-math-group.vala
 *
 * Copyright (C) 2018  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */
/**
 * A grouping math expression like parentesis
 */
public interface GCalc.MathGroup : Object, MathExpression {
  public enum Level {
    ONE,
    TWO,
    THREE
  }
  public abstract Level level { get; set; }
  public abstract bool closed { get; set; }
  public virtual MathExpression evaluate () throws GLib.Error {
    if (expressions.get_n_items () == 0) {
      throw new GroupError.INVALID_POLYNOMIAL (_("No internal polynomial in group"));
    }
    var e = expressions.get_item (0) as MathPolynomial;
    if (e == null) {
      throw new GroupError.INVALID_POLYNOMIAL (_("Invalid internal polynomial in group"));
    }
    return e.evaluate ();
  }
}

public errordomain GCalc.GroupError {
  INVALID_POLYNOMIAL,
  INVALID_INTERNAL_TERM,
}

